package com.gic.store.service.outer;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.dto.CityDTO;
import com.gic.store.dto.CountyDTO;
import com.gic.store.dto.ProvinceDTO;
import com.gic.store.service.ProvincesApiService;
import com.gic.store.service.ProvincesService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.concurrent.TimeUnit;

@Service("provincesApiService")
public class ProvincesApiServiceImpl implements ProvincesApiService {
    @Autowired
    private ProvincesService provincesService;

    @Override
    public ServiceResponse<List<ProvinceDTO>> selectAllProvince() {
        String key = "enterprise:province";
        List<ProvinceDTO> list = (List<ProvinceDTO>)RedisUtil.getCache(key);
        if(list == null){
            list = EntityUtil.changeEntityListByJSON(ProvinceDTO.class, this.provincesService.selectAllProvince());
            RedisUtil.setCache(key, list, 6*30l, TimeUnit.DAYS);
        }
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<CityDTO>> selectAllCity() {
        String key = "enterprise:city";
        List<CityDTO> list = (List<CityDTO>)RedisUtil.getCache(key);
        if(list == null){
            list = EntityUtil.changeEntityListByJSON(CityDTO.class, this.provincesService.selectAllCity());
            RedisUtil.setCache(key, list, 6*30l, TimeUnit.DAYS);
        }
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<CountyDTO>> selectAllCounty() {
        String key = "enterprise:county";
        List<CountyDTO> list = (List<CountyDTO>)RedisUtil.getCache(key);
        if(list == null){
            list = EntityUtil.changeEntityListByJSON(CountyDTO.class, this.provincesService.selectAllCounty());
            RedisUtil.setCache(key, list, 6*30l, TimeUnit.DAYS);
        }
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<ProvinceDTO>> queryProvinces(String search) {
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(ProvinceDTO.class, this.provincesService.queryProvinces(search)));
    }

    @Override
    public ServiceResponse<List<CityDTO>> queryCity(String search) {
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(CityDTO.class, this.provincesService.queryCity(search)));
    }

    @Override
    public ServiceResponse<List<CountyDTO>> queryCounty(String search) {
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(CountyDTO.class, this.provincesService.queryCounty(search)));
    }
}
