package com.gic.store.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.CreateTypeEnum;
import com.gic.store.dto.*;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreDictApiService;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.qo.store.StoreQO;
import com.gic.store.web.vo.StoreDictVO;
import com.gic.store.web.vo.StoreListSourceVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zhiwj
 * @date 2019/6/28
 */
@RestController
public class StoreController {

    private static final Logger logger = LogManager.getLogger(StoreController.class);

    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreDictApiService storeDictApiService;

    @RequestMapping("/save-store")
    public RestResponse saveOrUpdate(StoreQO storeQO) {
        Integer enterpriseId = 1111;
        StoreDTO storeDTO = EntityUtil.changeEntityByOrika(StoreDTO.class, storeQO);
        storeDTO.setEnterpriseId(enterpriseId);
        String photos = storeQO.getPhotos();
        if (StringUtils.isNotBlank(photos)) {
            List<StorePhotoDTO> photoList = JSON.parseArray(photos, StorePhotoDTO.class);
            storeDTO.setPhotoList(photoList);
        }
        String businessTimes = storeQO.getBusinessTimes();
        if (StringUtils.isNotBlank(businessTimes)) {
            List<StoreBusinessTimeDTO> businessTimeList = JSON.parseArray(businessTimes, StoreBusinessTimeDTO.class);
            storeDTO.setBusinessTimeList(businessTimeList);
        }
        storeDTO.setCreateType(CreateTypeEnum.BACKGROUND_ADD.getCode());
        String fieldJson = storeQO.getStoreExtends();
        List<StoreExtendDTO> extendList = JSON.parseArray(fieldJson, StoreExtendDTO.class);
        storeDTO.setStoreExtendList(extendList);
        ServiceResponse<Integer> serviceResponse = storeApiService.saveOrUpdate(storeDTO);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/store-detail")
    public RestResponse getStoreDetail(Integer storeId) {
        Integer enterpriseId = 1111;
        ServiceResponse<StoreDTO> serviceResponse = storeApiService.getStoreById(enterpriseId, storeId);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/store-list")
    public RestResponse storeList(StoreDTO storeDTO, PageQO pageQO) {
        ServiceResponse<Page<StoreDTO>> serviceResponse = storeApiService.listStore(storeDTO, pageQO.getCurrentPage(), pageQO.getPageSize());
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("store-list-source")
    @ResponseBody
    public RestResponse storeListSource(){
        Integer enterpriseId = 1111;
        List<StoreListSourceVO> list = this.getStoreAllListSource(enterpriseId);
        Object obj = RedisUtil.getCache("enterprise:storeListSources:"+enterpriseId);
        if(obj == null){
            return RestResponse.success(list);
        }
        String[] sourcesArr = obj.toString().split(",");
        for(StoreListSourceVO storeListSourceVO : list){
            for(String s : sourcesArr){
                if(s.equals(storeListSourceVO.getKey())){
                    storeListSourceVO.setChecked(1);
                }
            }
        }
        return RestResponse.success(list);
    }

    @RequestMapping("store-list-source-set")
    @ResponseBody
    public RestResponse storeListSourceSet(String sources){
        Integer enterpriseId = 1111;
        RedisUtil.setCache("enterprise:storeListSources:"+enterpriseId, sources);
        return RestResponse.success();
    }

    private List<StoreListSourceVO> getStoreAllListSource(Integer enterpriseId){
        List<StoreListSourceVO> list = new ArrayList<>();
        ServiceResponse<List<StoreDictDTO>> serviceResponse = this.storeDictApiService.listStoreField(enterpriseId);
        if(serviceResponse.isSuccess()){
            List<StoreDictDTO> storeDictDTOList = serviceResponse.getResult();
            if(CollectionUtils.isNotEmpty(storeDictDTOList)){
                for(StoreDictDTO storeDictDTO : storeDictDTOList){
                    if(storeDictDTO.getChecked() == 1){
                        StoreListSourceVO vo = new StoreListSourceVO();
                        vo.setChecked(0);
                        vo.setIsCustom(1);
                        vo.setKey(storeDictDTO.getKey());
                        vo.setValue(storeDictDTO.getValue());
                        list.add(vo);
                    }
                }
            }
        }
        StoreListSourceVO vo1 = new StoreListSourceVO();
        vo1.setValue("storeName");
        vo1.setKey("门店名称");
        list.add(vo1);
        StoreListSourceVO vo2 = new StoreListSourceVO();
        vo2.setValue("storeGroupName");
        vo2.setKey("门店分组名称");
        list.add(vo2);
        StoreListSourceVO vo3 = new StoreListSourceVO();
        vo3.setValue("storeCode");
        vo3.setKey("关联门店主键");
        list.add(vo3);
        StoreListSourceVO vo4 = new StoreListSourceVO();
        vo4.setValue("provinces");
        vo4.setKey("地区");
        list.add(vo4);
        StoreListSourceVO vo5 = new StoreListSourceVO();
        vo5.setValue("brandNames");
        vo5.setKey("门店品牌");
        list.add(vo5);
        StoreListSourceVO vo6 = new StoreListSourceVO();
        vo6.setValue("completeStatus");
        vo6.setKey("资料完善状态");
        list.add(vo6);
        StoreListSourceVO vo7 = new StoreListSourceVO();
        vo7.setValue("status");
        vo7.setKey("门店启用状态");
        list.add(vo7);
        StoreListSourceVO vo8 = new StoreListSourceVO();
        vo8.setValue("regionName");
        vo8.setKey("门店所属域");
        list.add(vo8);
        StoreListSourceVO vo9 = new StoreListSourceVO();
        vo9.setValue("erpStatus");
        vo9.setKey("门店状态");
        list.add(vo9);
        StoreListSourceVO vo10 = new StoreListSourceVO();
        vo10.setValue("storeType");
        vo10.setKey("门店类型");
        list.add(vo10);
        return list;
    }

}