package com.gic.store.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.store.constant.CreateTypeEnum;
import com.gic.store.constant.StoreFieldTypeEnum;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.constant.StrategyStoreFieldEnum;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.dto.StoreFieldSelectDTO;
import com.gic.store.dto.StoreStrategyDTO;
import com.gic.store.service.StoreFieldApiService;
import com.gic.store.service.StoreFieldSelectApiService;
import com.gic.store.service.StoreStrategyApiService;
import com.gic.store.utils.ErrorCode;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.qo.storestrategy.StoreStrategyQO;
import com.gic.store.web.vo.storestrategy.StoreGroupStrateVO;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author guojx
 * @date 2019/7/4 1:56 PM
 */
@RestController
@RequestMapping("/store-strategy")
public class StoreStrategyController {

    private static final Logger logger = LoggerFactory.getLogger(StoreStrategyController.class);

    @Autowired
    private StoreStrategyApiService storeStrategyApiService;

    @Autowired
    private StoreFieldApiService storeFieldApiService;
    @Autowired
    private StoreFieldSelectApiService storeFieldSelectApiService;

    @RequestMapping("/list-store-field")
    public RestResponse listStoreField(Integer regionId) {
        if (regionId == null) {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), "门店域ID不能为空");
        }
        Map<String, String> result = new HashMap<>(16);
        for (StrategyStoreFieldEnum fieldEnum : StrategyStoreFieldEnum.values()) {
            result.put(fieldEnum.getCode(), fieldEnum.getMessage());
        }
        //单选或者多选的，也需要
        ServiceResponse<List<StoreFieldDTO>> storeFieldDTOServiceResponse = storeFieldApiService.listStoreFieldByRegionId(regionId);
        if (storeFieldDTOServiceResponse.isSuccess()) {
            List<StoreFieldDTO> storeFieldDTOList = storeFieldDTOServiceResponse.getResult();
            for (StoreFieldDTO storeFieldDTO : storeFieldDTOList) {
                int storeFieldType = storeFieldDTO.getStoreFieldType();
                boolean isRadioOrCheck = storeFieldType == StoreFieldTypeEnum.RADIO.getCode() ||
                        storeFieldType == StoreFieldTypeEnum.CHECK.getCode();
                if (isRadioOrCheck) {
                    result.put(storeFieldDTO.getStoreFieldId().toString(), storeFieldDTO.getStoreFieldName());
                }
            }
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/list-store-field-select")
    public RestResponse listStoreFieldSelect(Integer storeFieldId) {
        if (storeFieldId == null) {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), "单选或者多选自定义不能为空");
        }
        Map<Integer, String> result = new HashMap<>(16);
        ServiceResponse<Page<StoreFieldSelectDTO>> serviceResponse = storeFieldSelectApiService.listStoreFieldSelect(storeFieldId, null,
                1, Integer.MAX_VALUE);
        if (serviceResponse.isSuccess()) {
            List<StoreFieldSelectDTO> selectDTOList = serviceResponse.getResult().getResult();
            for (StoreFieldSelectDTO dto : selectDTOList) {
                result.put(dto.getStoreFieldSelectId(), dto.getStoreFieldSelectName());
            }
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/save-store-strategy")
    public RestResponse saveStoreStrategy(@Validated({StoreStrategyQO.SaveValidView.class, StoreStrategyQO.StoreStrategyTypeValidView.class})
                                                           StoreStrategyQO storeStrategyQO) {
        ServiceResponse result = storeStrategyApiService.saveStoreStrategy(transferQoToDTO(storeStrategyQO));
        if (!result.isSuccess()) {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
        return RestResponse.success(result.getResult());
    }

    @RequestMapping("/list-store-create-type")
    public RestResponse listStoreCreateType() {
        Map<Integer, String> result = new HashMap<>(16);
        for (CreateTypeEnum createTypeEnum : CreateTypeEnum.values()) {
            result.put(createTypeEnum.getCode(), createTypeEnum.getMessage());
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/reset-store-group-strategy")
    public RestResponse resetStoreGroupStrategy(String regionIds, String status, boolean isDealWith) {
        if (isDealWith) {
            if (StringUtils.isBlank(regionIds)) {
                return RestResponse.failure(ErrorCode.ERR_5.getCode(), "门店域不能为空");
            }
            if (StringUtils.isBlank(status)) {
                return RestResponse.failure(ErrorCode.ERR_5.getCode(), "门店启用状态不能为空");
            }

            ServiceResponse result = storeStrategyApiService.resetStoreGroupStrategy(transferStringArrToList(regionIds), transferStringArrToList(status));
            if (!result.isSuccess()) {
                return RestResponse.failure(result.getCode(), result.getMessage());
            }
        }

        return RestResponse.success();
    }


    @RequestMapping("/edit-store-strategy")
    public RestResponse editStoreStrategy(@Validated({StoreStrategyQO.SaveValidView.class, StoreStrategyQO.RemoveValidView.class})
                                                           StoreStrategyQO storeStrategyQO) {
        ServiceResponse result = storeStrategyApiService.modifyStoreStrategy(transferQoToDTO(storeStrategyQO));
        if (!result.isSuccess()) {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
        return RestResponse.success(result.getResult());
    }

    @RequestMapping("/get-store-strategy")
    public RestResponse getStoreStrategy(@Validated({StoreStrategyQO.RemoveValidView.class}) StoreStrategyQO storeStrategyQO) {
        ServiceResponse result = storeStrategyApiService.getStoreStrategyById(storeStrategyQO.getStrategyId());
        if (!result.isSuccess()) {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
        return RestResponse.success(result.getResult());
    }

    @RequestMapping("/delete-store-strategy")
    public RestResponse deleteStoreStrategy(@Validated({StoreStrategyQO.RemoveValidView.class}) StoreStrategyQO storeStrategyQO) {
        ServiceResponse result = storeStrategyApiService.deleteStoreStrategy(storeStrategyQO.getStrategyId());
        if (!result.isSuccess()) {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
        return RestResponse.success(result.getResult());
    }


    @RequestMapping("/swap-store-strategy")
    public RestResponse swapStoreStrategy(@Validated({StoreStrategyQO.SwapValidView.class, StoreStrategyQO.RemoveValidView.class}) StoreStrategyQO storeStrategyQO) {
        int upOrDown = storeStrategyQO.getUpOrDown();
        ServiceResponse result;
        switch (upOrDown) {
            case StoreGroupConstant.SORT_UP:
                result = storeStrategyApiService.upOrDown(storeStrategyQO.getStrategyId(), true);
                break;
            case StoreGroupConstant.SORT_DOWN:
                result = storeStrategyApiService.upOrDown(storeStrategyQO.getStrategyId(), false);
                break;
            case StoreGroupConstant.SORT_BOTTOM:
                result = storeStrategyApiService.sortBottom(storeStrategyQO.getStrategyId());
                break;
            case StoreGroupConstant.SORT_TOPPING:
                result = storeStrategyApiService.sortTopping(storeStrategyQO.getStrategyId());
                break;
            default:
                result = null;
        }
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/set-store-strategy-sort-value")
    public RestResponse setStoreStrategySortValue(@Validated({StoreStrategyQO.SetSortValueValidView.class, StoreStrategyQO.RemoveValidView.class}) StoreStrategyQO storeStrategyQO) {
        ServiceResponse result = storeStrategyApiService.setStoreStrategySort(storeStrategyQO.getStrategyId(), storeStrategyQO.getSortValue());
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/list-store-strategy")
    public RestResponse listStoreStrategy(String search, PageQO pageQO, @Validated({StoreStrategyQO.StoreStrategyTypeValidView.class}) StoreStrategyQO storeStrategyQO) {
        ServiceResponse<Page<StoreStrategyDTO>> result = storeStrategyApiService.listStoreField(StoreGroupConstant.TEST_ENTERPRISE_ID
                , search, storeStrategyQO.getStrategyType(), pageQO.getCurrentPage(), pageQO.getPageSize());
        if (result.isSuccess()) {
            Page<StoreStrategyDTO> resultPage = result.getResult();
            Page<StoreGroupStrateVO> voPage = EntityUtil.changeEntityNew(Page.class, resultPage);
            voPage.setResult(EntityUtil.changeEntityListNew(StoreGroupStrateVO.class, resultPage.getResult()));
            return RestResponse.success(voPage);
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/update-store-strategy-weights")
    public RestResponse updateWeight(Integer weight) {
        if (weight == null) {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), "参数错误");
        }
        ServiceResponse<Integer> result = storeStrategyApiService.setStoreGroupStrategyWeight(StoreGroupConstant.TEST_ENTERPRISE_ID, weight);
        return RestResponse.success(result.getResult());
    }

    @RequestMapping("/get-store-strategy-weights")
    public RestResponse getWeight() {
        ServiceResponse<Integer> result = storeStrategyApiService.getStoreGroupStrategyWeight(StoreGroupConstant.TEST_ENTERPRISE_ID);
        return RestResponse.success(result.getResult());
    }


    private StoreStrategyDTO transferQoToDTO(StoreStrategyQO storeStrategyQO) {
        StoreStrategyDTO storeStrategyDTO = EntityUtil.changeEntityNew(StoreStrategyDTO.class, storeStrategyQO);
        storeStrategyDTO.setEnterpriseId(StoreGroupConstant.TEST_ENTERPRISE_ID);
        return storeStrategyDTO;
    }


    private List<Integer> transferStringArrToList(String arr) {
        String[] strArr = arr.split(",");
        List<Integer> list = new ArrayList<>(strArr.length);
        for (String str : strArr) {
            list.add(Integer.parseInt(str));
        }
        return list;
    }
}
